
/////////////////////////////////////////////////////////////////////
// FUNCIONES PARA EL MANEJO DE MEMORIA
/////////////////////////////////////////////////////////////////////

// Reserva memoria para un arreglo unidimensional
// rows  - Numero de filas
template<class T>
T *array1D(int rows) {
    T *vec=NULL;
    vec = new T[rows];
    return(vec);
}


// Reserva memoria para un arreglo bidimensional
// rows  - Numero de filas
// cols  - Numero de columnas
template<class T>
T **array2D(int rows, int cols) {
    int    i;
    T   **mat=NULL;
    
    // Reservamos memoria para apuntadores a cada fila de la matriz
    mat     = new T*[rows];       
    // Reservamos la memoria para todas las entradas
    mat[0]  = new T[rows*cols];
    // Inicializamos el resto de los apuntadores
    for(i=1; i<rows; i++)  mat[i] = mat[i-1] + cols;
    return(mat);
}

// Funcion para leer un archivo que contiene los datos de una matriz,
// en el formato acordado. Usamos un template pensando que 
// podriamos usar cualquier tipo de datos, aunque usualmente
// seran datos de tipo double.
// Devuelve un doble apuntador a los datos y guarda las dimensiones
// de la matriz en las variables rows y cols.
template<class T>
T **readBinMatrix(char *cname, int &rows, int &cols)  {
    T    **mat;
    FILE  *fp = fopen(cname, "rb");

    if(fp==NULL) {
       printf("Error de lectura del archivo %s\n", cname); 
       rows = cols = 0;
       return(NULL);
    }
    fread(&rows, sizeof(int), 1, fp); // Leemos el numero de filas
    fread(&cols, sizeof(int), 1, fp); // Leemos el numero de columnas

    // Reservamos memoria para la matriz
    mat = array2D<T>(rows, cols);

    // La ventaja de reservar la memoria como lo hace la funcion array2D()
    // es que podemos ver los datos como un arreglo bidimensional usando 
    // la variable mat o verlos como un arreglo unidimensional usando mat[0]
    fread(mat[0], sizeof(T), rows*cols, fp);

    fclose(fp);
    return(mat);
}

// Funcion para leer un archivo que contiene los entradas de un vector. 
// Devuelve un apuntados a los datos y almacena en la variable
// rows la dimension del vector
template<class T>
T *readBinVector(char *cname, int &rows)  {
    int    i;
    T     *vec;
    FILE  *fp = fopen(cname, "rb");

    if(fp==NULL) {
       printf("Error de lectura del archivo %s\n", cname); 
       rows  = 0;
       return(NULL);
    }
    fread(&rows, sizeof(int), 1, fp); // Numero de filas
    fread(&i,    sizeof(int), 1, fp); // Numero de columnas. Debe ser igual a 1
    vec  = array1D<T>(rows);
    fread(vec, sizeof(T), rows, fp);
    fclose(fp);
    return(vec);
}

// Funcion para imprimir en consola un arreglo unidimensional
// como si fuera una matriz. Las entradas son
// cft   - Cadena con el formato para imprimir cada entrada de la matriz
// rows  - Numero de filas de la matriz
// cols  - Numero de columnas de la matriz
// array - Arreglo unimensional con los datos 
template<class T>
void printMatrix(char *cft, int rows, int cols, T *array) {
    int   i, j;
    T    *ptr = array;
    for(i=0; i<rows; ++i) {
	for(j=0; j<cols; ++j, ++ptr) printf(cft, *ptr);
	printf("\n");
    }
    printf("\n");
}

// Funciones para liberar memoria
template <class T>
void freeMatrix(double **mat) {
     delete[]  mat[0];
     delete[]  mat;  
}

template <class T>
void freeVector(double *vec) {
     delete[]  vec;  
}



