#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "memoria.c"


/////////////////////////////////////////////////////////////////////
// ALGORITMO
/////////////////////////////////////////////////////////////////////

// En este caso el algoritmo solo es la multiplicacion de una matriz
// por un vector. Las entradas son
// mat   - apuntador al arreglo de la matriz
// vec   - apuntador al vector
// nrows - Numero de filas de la matriz
// ncols - Numero de columnas de la matriz
// El resultado se almacena en la direccion especificada por el 
// apuntados res.
template<class T>
void multiplica(double **mat, double *vec, int nrows, int ncols, double *res) {
    int   i, j;
    T     val;
    for(i=0; i<nrows; ++i) {
	val = 0;
	for(j=0; j<ncols; ++j) 
	    val += mat[i][j]*vec[j];
	res[i] = val;
    }
}

// Funcion que lee los datos y aplica el procedimiento indicado.
// Los datos de entrada son:
// cmat   - Nombre del archivo con las entradas de la matriz
// cvec   - Nombre del archivo con las entradas del vector
void procedimiento(char *cmat, char *cvec) {
    int       rows, cols, dim;
    double  **mat, *vec, *res;

    mat = readBinMatrix<double>(cmat, rows, cols);
    vec = readBinVector<double>(cvec, dim);
    res = new double[rows];
       
    if(dim!=rows) {
	printf("El numero de columnas de la matriz es diferente\n");
        printf("de la dimension del vector\n");
	exit(0);
    }
    printf("Matriz de dimension %d x %d\n", rows, cols);
    printf("Vector de dimension %d\n", dim);

    printf("Vector v: \n");
    printMatrix<double>("% 10.6f ", 1, dim, vec);

    // Ahora podemos hacer uso de que podemos accesar a los 
    // elementos de la matriz como si fuera un arreglo unidimensional
    // y poder usar la funcion printMatrix para imprimir los datos
    printf("Matriz A: \n");
    printMatrix<double>("% 10.6f ", rows, cols, mat[0]);

    // Realizamos el producto de la matriz por el vector
    multiplica<double>(mat, vec, rows, cols, res);

    // Imprimimos el resultado
    printf("Resultado A*v: \n");
    printMatrix<double>("% 10.6f ", 1, rows, res);


    freeMatrix<double>(mat);
    freeVector<double>(vec);
    freeVector<double>(res);
}


/////////////////////////////////////////////////////////////////////
// FUNCION PRINCIPAL
/////////////////////////////////////////////////////////////////////

int main (int argc, char *argv[]) {

    if(argc!=3) {
	printf("\nFORMA DE INVOCAR EL PROGRAMA:\n");
	printf("\tmulMatVec cmat cvec\n");
	printf("donde cmat es el nombre del archivo binario con las entradas\n");
	printf("una matriz y cvec es el nombre de un archivo binario con las\n");
	printf("entradas de un vector. El tipo de dato de las entradas debe ");
	printf("ser double\n\n");
	return(0);
    }

    procedimiento(argv[1], argv[2]);
   
    printf("Listo!\n");

    return(0);
}



